"""
pyrad.io.config
===============

Functions for reading pyrad config files

.. autosummary::
    :toctree: generated/

    read_config
    get_num_elements
    string_to_datatype
    get_array
    get_struct
    get_array_type
    init_array

"""

_defaults_loc = {
    "ppiImageConfig": {
        "xsize": 10,
        "ysize": 10,
        "dpi": 72,
        "xmin": -50,
        "xmax": 50,
        "ymin": -50,
        "ymax": 50,
    },
    "ppiMapImageConfig": {"xsize": 10, "ysize": 10, "dpi": 72},
    "rhiImageConfig": {
        "xsize": 10,
        "ysize": 5,
        "dpi": 72,
        "xmin": 0,
        "xmax": 50,
        "ymin": 0,
        "ymax": 12,
    },
    "xsecImageConfig": {
        "xsize": 10,
        "ysize": 5,
        "dpi": 72,
        "xmin": 0,
        "xmax": 50,
        "ymin": 0,
        "ymax": 12,
    },
    "gridMapImageConfig": {"xsize": 10, "ysize": 10, "dpi": 72},
    "sunhitsImageConfig": {
        "xsize": 10,
        "ysize": 5,
        "dpi": 72,
        "azmin": -2.0,
        "azmax": 2.0,
        "elmin": -2.0,
        "elmax": 2.0,
        "azres": 0.1,
        "elres": 0.1,
    },
    "spectraImageConfig": {
        "xsize": 10,
        "ysize": 5,
        "dpi": 72,
        "ymin": None,
        "ymax": None,
        "velmin": None,
        "velmax": None,
    },
    "ScanPeriod": 5,
    "IconRunFreq": 3,
    "IconForecasted": 7,
    "RadarName": None,
    "RadarRes": None,
    "NumRadars": 1,
    "TimeTol": 3600.0,
    "ScanList": None,
}

_defaults_main = {
    "lastStateFile": None,
    "datapath": None,
    "satpath": None,
    "iconpath": None,
    "psrpath": None,
    "iqpath": None,
    "configpath": None,
    "colocgatespath": None,
    "excessgatespath": None,
    "dempath": None,
    "smnpath": None,
    "disdropath": None,
    "solarfluxpath": None,
    "selfconsistencypath": None,
    "loadbasepath": None,
    "loadname": None,
    "gecsxbasepath": None,
    "gecsxname": None,
    "metranet_read_lib": "C",
}

DEFAULT_CONFIG = {"loc": _defaults_loc, "main": _defaults_main}



"""
pyrad.prod.process_intercomp_products
=====================================

Functions for obtaining Pyrad products from datasets used in the
intercomparison process

.. autosummary::
    :toctree: generated/

    generate_intercomp_products
    generate_colocated_gates_products
    generate_time_avg_products

"""

from ..util import warn

import numpy as np

from .process_vol_products import generate_vol_products

from ..io.io_aux import get_fieldname_pyart
from ..io.io_aux import get_save_dir, make_filename

from ..io.read_data_other import read_intercomp_scores_ts

from ..io.write_data import write_colocated_gates, write_colocated_data
from ..io.write_data import write_colocated_data_time_avg
from ..io.write_data import write_intercomp_scores_ts

from ..graph.plots import plot_scatter
from ..graph.plots_timeseries import plot_intercomp_scores_ts
from ..graph import plot_colocated_gates

from ..util.radar_utils import compute_2d_stats
from ..util.stat_utils import parse_math_expression


def generate_intercomp_products(dataset, prdcfg):
    """
    Generates radar intercomparison products. Accepted product types:
        'PLOT_AND_WRITE_INTERCOMP_TS': Writes statistics of radar
            intercomparison in a file and plots the time series of the
            statistics. Note that in order to use this product you need
            to also define the product WRITE_INTERCOMP_TIME_AVG for the dataset in question.
            User defined parameters:
                voltype: str
                    name of the pyrad variable to use, it must be available in the dataset
                add_date_in_fname: Bool
                    If true adds the year in the csv file containing the
                    statistics. Default False
                sort_by_date: Bool
                    If true sorts the statistics by date when reading the
                    csv file containing the statistics. Default False
                step: float
                    The quantization step of the data. If none it will be
                    computed using the Py-ART config file. Default None
                rewrite: Bool
                    If true rewrites the csv file containing the statistics.
                    Default False
                npoints_min: int
                    The minimum number of points to consider the statistics
                    valid and therefore use the data point in the plotting.
                    Default 0
                transform: str
                    A transform to apply to the data before computing the statistics
                    and generating the plots. Any mathematical function with argument "x"
                    is accepted for example "sin(x) + x**2" or "10 * log10(x)"
                    Default is to use no transform
                corr_min: float
                    The minimum correlation to consider the statistics
                    valid and therefore use the data point in the plotting.
                    Default 0.
        'PLOT_SCATTER_INTERCOMP': Plots a density plot (2D histogram) with the points of
            radar 1 versus the points of radar 2. Note that in order to use this product you need
            to also define the product WRITE_INTERCOMP_TIME_AVG for the dataset in question.
            User defined parameters:
                voltype: str
                    name of the pyrad variable to use, it must be available in the dataset
                step: float
                    The quantization step of the data. If none it will be
                    computed using the Py-ART config file. Default None
                vmin: float
                    Minimum value of the density plot. If vmin or vmax are not define the range limits
                    of the field as defined in the Py-ART config file are going to be used.
                vmax: float
                    Maximum value of the density plot. If vmin or vmax are not define the range limits
                    of the field as defined in the Py-ART config file are going to be used.
                cap_limits: bool
                    If set to 1, all values larger than vmax or smaller than vmin will be discarded from the
                    scatter plot. If set to 0, they will be kept and assigned to the smallest/largest bin
                    of the histogram. Default is 0.
                scatter_type: str
                    Type of scatter plot. Can be a plot for each radar volume
                    (instant) or at the end of the processing period
                    (cumulative). Default is cumulative
                range_bins: list of floats
                    Range bins to radar 1 to consider, a different plot will be generated for
                    all range bins.This can be used to differentiate the analysis in different
                    radar ranges.
                    Default is to generate only one figure, regardless of the distance to radar 1.
                transform: str
                    A transform to apply to the data before computing the statistics
                    and generating the plots. Any mathematical function with argument "x"
                    is accepted for example "sin(x) + x**2" or "10 * log10(x)"
                    Default is to use no transform
        'WRITE_INTERCOMP': Writes the instantaneously intercompared data
            (gate positions, values, etc.) in a csv file.
            User defined parameters:
                voltype: str
                    name of the pyrad variable to use, it must be available in the dataset
        'WRITE_INTERCOMP_TIME_AVG': Writes the time-averaged intercompared
            data (gate positions, values, etc.) in a csv file.
            User defined parameters:
                voltype: str
                    name of the pyrad variable to use, it must be available in the dataset

    Parameters
    ----------
    dataset : tuple
        values of colocated gates dictionary

    prdcfg : dictionary of dictionaries
        product configuration dictionary of dictionaries

    Returns
    -------
    filename : str
        the name of the file created. None otherwise

    """

    dssavedir = prdcfg["dsname"]
    if "dssavename" in prdcfg:
        dssavedir = prdcfg["dssavename"]

    if prdcfg["type"] == "WRITE_INTERCOMP":
        if dataset["final"]:
            return None

        savedir = get_save_dir(
            prdcfg["basepath"],
            prdcfg["procname"],
            dssavedir,
            prdcfg["prdname"],
            timeinfo=dataset["timeinfo"],
        )

        fname = make_filename(
            "colocated_data",
            prdcfg["dstype"],
            prdcfg["voltype"],
            ["csv"],
            timeinfo=dataset["timeinfo"],
            timeformat="%Y%m%d",
        )

        fname = savedir + fname[0]
        write_colocated_data(dataset["intercomp_dict"], fname)

        print("saved colocated data file: " + fname)

        return fname

    if prdcfg["type"] == "WRITE_INTERCOMP_TIME_AVG":
        if dataset["final"]:
            return None

        savedir = get_save_dir(
            prdcfg["basepath"],
            prdcfg["procname"],
            dssavedir,
            prdcfg["prdname"],
            timeinfo=dataset["timeinfo"],
        )

        fname = make_filename(
            "colocated_data",
            prdcfg["dstype"],
            prdcfg["voltype"],
            ["csv"],
            timeinfo=dataset["timeinfo"],
            timeformat="%Y%m%d",
        )
        fname = savedir + fname[0]
        write_colocated_data_time_avg(dataset["intercomp_dict"], fname)
        print("saved colocated time averaged data file: " + fname)

        return fname

    if prdcfg["type"] == "PLOT_SCATTER_INTERCOMP":
        scatter_type = prdcfg.get("scatter_type", "cumulative")
        if scatter_type == "cumulative" and not dataset["final"]:
            return None

        timeformat = "%Y%m%d"
        if scatter_type == "instant":
            timeformat = "%Y%m%d%H%M%S"
            colname = "dBZavg"
        else:
            colname = "val"
        if f"rad1_{colname}" not in dataset["intercomp_dict"]:
            return

        field_name = get_fieldname_pyart(prdcfg["voltype"])
        savedir = get_save_dir(
            prdcfg["basepath"],
            prdcfg["procname"],
            dssavedir,
            prdcfg["prdname"],
            timeinfo=dataset["timeinfo"],
        )

        transform_str = prdcfg.get("transform", "x")
        transform = parse_math_expression(transform_str)
        range_bins = prdcfg.get("range_bins", [0, np.inf])
        step = prdcfg.get("step", None)
        vmin = prdcfg.get("vmin", None)
        vmax = prdcfg.get("vmax", None)
        cap_limits = prdcfg.get("cap_limits", 0)

        fname_list = []
        for i in range(len(range_bins) - 1):  # loop on range bins
            if len(range_bins) > 2:
                rangebin_info = f"range_bin_{range_bins[i]:.0f}_{range_bins[i+1]:.0f}m"
                rangebin_info_title = (
                    f"range_bin {range_bins[i]:.0f}-{range_bins[i+1]:.0f}m"
                )
            else:
                rangebin_info = None  # only one range bin, leave empty
                rangebin_info_title = ""

            f_list = make_filename(
                "scatter",
                prdcfg["dstype"],
                prdcfg["voltype"],
                prdcfg["imgformat"],
                prdcfginfo=rangebin_info,
                timeinfo=dataset["timeinfo"],
                timeformat=timeformat,
            )

            for j, fname in enumerate(f_list):
                f_list[j] = savedir + fname

            rad1 = dataset["intercomp_dict"]["rad1_name"]
            rad2 = dataset["intercomp_dict"]["rad2_name"]
            titl = (
                f"colocated radar gates {rad1}-{rad2} \n {rangebin_info_title} "
                + dataset["timeinfo"].strftime(timeformat)
            )

            selection = np.logical_and(
                dataset["intercomp_dict"]["rad1_rng"] >= range_bins[i],
                dataset["intercomp_dict"]["rad1_rng"] < range_bins[i + 1],
            )
            if not np.sum(selection):  # skip if selection empty
                continue

            hist_2d, bin_edges1, bin_edges2, stats = compute_2d_stats(
                np.ma.asarray(dataset["intercomp_dict"][f"rad1_{colname}"][selection]),
                np.ma.asarray(dataset["intercomp_dict"][f"rad2_{colname}"][selection]),
                field_name,
                field_name,
                step1=step,
                step2=step,
                transform=transform,
                vmin=vmin,
                vmax=vmax,
                cap_limits=cap_limits,
            )
            if hist_2d is None:
                return None

            metadata = (
                "npoints: "
                + str(stats["npoints"])
                + "\n"
                + "mode bias: "
                + "{:.2f}".format(float(stats["modebias"]))
                + "\n"
                + "median bias: "
                + "{:.2f}".format(float(stats["medianbias"]))
                + "\n"
                + "mean bias: "
                + "{:.2f}".format(float(stats["meanbias"]))
                + "\n"
                + "intercep slope 1: "
                + "{:.2f}".format(float(stats["intercep_slope_1"]))
                + "\n"
                + "corr: "
                + "{:.2f}".format(float(stats["corr"]))
                + "\n"
                + "slope: "
                + "{:.2f}".format(float(stats["slope"]))
                + "\n"
                + "intercep: "
                + "{:.2f}".format(float(stats["intercep"]))
                + "\n"
            )
            if transform_str != "x":
                field_name = f"{transform_str} of {field_name}"
            plot_scatter(
                bin_edges1,
                bin_edges2,
                np.ma.asarray(hist_2d),
                field_name,
                field_name,
                f_list,
                prdcfg,
                titl=titl,
                metadata=metadata,
                lin_regr=[stats["slope"], stats["intercep"]],
                lin_regr_slope1=stats["intercep_slope_1"],
                rad1_name=dataset["intercomp_dict"]["rad1_name"],
                rad2_name=dataset["intercomp_dict"]["rad2_name"],
                vmin=vmin,
                vmax=vmax,
            )

            fname_list.extend(f_list)

            print("----- save to " + " ".join(f_list))

        return fname_list

    if prdcfg["type"] == "PLOT_AND_WRITE_INTERCOMP_TS":
        if not dataset["final"]:
            return None

        field_name = get_fieldname_pyart(prdcfg["voltype"])
        step = prdcfg.get("step", None)
        transform = parse_math_expression(prdcfg.get("transform", "x"))
        rad1_name = dataset["intercomp_dict"]["rad1_name"]
        rad2_name = dataset["intercomp_dict"]["rad2_name"]

        hist_2d, bin_edges1, bin_edges2, stats = compute_2d_stats(
            transform(np.ma.asarray(dataset["intercomp_dict"]["rad1_val"])),
            transform(np.ma.asarray(dataset["intercomp_dict"]["rad2_val"])),
            field_name,
            field_name,
            step1=step,
            step2=step,
        )

        # put time info in file path and name
        csvtimeinfo_file = None
        timeformat = None
        sort_by_date = False
        rewrite = False
        if "add_date_in_fname" in prdcfg:
            if prdcfg["add_date_in_fname"]:
                csvtimeinfo_file = dataset["timeinfo"]
                timeformat = "%Y"
        if "sort_by_date" in prdcfg:
            sort_by_date = prdcfg["sort_by_date"]
        if "rewrite" in prdcfg:
            rewrite = prdcfg["rewrite"]

        savedir = get_save_dir(
            prdcfg["basepath"],
            prdcfg["procname"],
            dssavedir,
            prdcfg["prdname"],
            timeinfo=None,
        )

        csvfname = make_filename(
            "ts",
            prdcfg["dstype"],
            prdcfg["voltype"],
            ["csv"],
            prdcfginfo=rad1_name + "-" + rad2_name,
            timeinfo=csvtimeinfo_file,
            timeformat=timeformat,
        )[0]

        csvfname = savedir + csvfname

        write_intercomp_scores_ts(
            dataset["timeinfo"],
            stats,
            field_name,
            csvfname,
            rad1_name=rad1_name,
            rad2_name=rad2_name,
        )
        print("saved CSV file: " + csvfname)

        (
            date_vec,
            np_vec,
            meanbias_vec,
            medianbias_vec,
            quant25bias_vec,
            quant75bias_vec,
            modebias_vec,
            corr_vec,
            slope_vec,
            intercep_vec,
            intercep_slope1_vec,
        ) = read_intercomp_scores_ts(csvfname, sort_by_date=sort_by_date)

        if date_vec is None:
            warn("Unable to plot time series. No valid data")
            return None

        if len(date_vec) < 2:
            warn("Unable to plot time series. Not enough points")
            return None

        if rewrite:
            stats = {
                "npoints": np_vec,
                "meanbias": meanbias_vec,
                "medianbias": medianbias_vec,
                "quant25bias": quant25bias_vec,
                "quant75bias": quant75bias_vec,
                "modebias": modebias_vec,
                "corr": corr_vec,
                "slope": slope_vec,
                "intercep": intercep_vec,
                "intercep_slope_1": intercep_slope1_vec,
            }
            write_intercomp_scores_ts(
                date_vec,
                stats,
                field_name,
                csvfname,
                rad1_name=rad1_name,
                rad2_name=rad2_name,
            )

        figtimeinfo = None
        titldate = (
            date_vec[0].strftime("%Y%m%d") + "-" + date_vec[-1].strftime("%Y%m%d")
        )
        if "add_date_in_fname" in prdcfg:
            if prdcfg["add_date_in_fname"]:
                figtimeinfo = date_vec[0]
                timeformat = "%Y"

        figfname_list = make_filename(
            "ts",
            prdcfg["dstype"],
            prdcfg["voltype"],
            prdcfg["imgformat"],
            prdcfginfo=rad1_name + "-" + rad2_name,
            timeinfo=figtimeinfo,
            timeformat=timeformat,
        )

        for i, figfname in enumerate(figfname_list):
            figfname_list[i] = savedir + figfname

        np_min = prdcfg.get("npoints_min", 0)
        corr_min = prdcfg.get("corr_min", 0.0)

        titl = (
            rad1_name
            + "-"
            + rad2_name
            + " "
            + field_name
            + " intercomparison "
            + titldate
        )
        plot_intercomp_scores_ts(
            date_vec,
            np_vec,
            meanbias_vec,
            medianbias_vec,
            quant25bias_vec,
            quant75bias_vec,
            modebias_vec,
            corr_vec,
            slope_vec,
            intercep_vec,
            intercep_slope1_vec,
            figfname_list,
            ref_value=0.0,
            np_min=np_min,
            corr_min=corr_min,
            labelx="Time UTC",
            titl=titl,
        )
        print("----- save to " + " ".join(figfname_list))

        return figfname_list

    warn(" Unsupported product type: " + prdcfg["type"])
    return None


def generate_colocated_gates_products(dataset, prdcfg):
    """
    Generates colocated gates products. Accepted product types:
        'WRITE_COLOCATED_GATES': Writes the position of the co-located gates
            in a csv file
        'PLOT_COLOCATED_GATES': Plots the colocated gates over a geo-referenced map,
            as well as the coordinates of both radars. The plot performs a gridding and
            plots the number of cologates gates within every grid-cell as well as their
            average height. All other plotting parameters are obtained from the
            gridMapImageConfig structure of the loc file.
            User defined parameters:
                grid_size: float
                    Size of the grid in degrees (lat/lon). If not provided,
                    the grid size defined in gridMapImageConfig with
                    lat_min, lon_min, lat_max, lon_max and lonstep, latstep
                    will be used.
        All the products of the 'VOL' dataset group


    Parameters
    ----------
    dataset : tuple
        radar objects and colocated gates dictionary

    prdcfg : dictionary of dictionaries
        product configuration dictionary of dictionaries

    Returns
    -------
    filename : str
        the name of the file created. None otherwise

    """
    if prdcfg["type"] == "WRITE_COLOCATED_GATES":
        if prdcfg["radar"] not in dataset:
            return None
        if "coloc_dict" not in dataset[prdcfg["radar"]]:
            return None

        savedir = get_save_dir(
            prdcfg["basepath"],
            prdcfg["procname"],
            "colocated_gates",
            prdcfg["prdname"],
            timeinfo=None,
        )

        fname = make_filename(
            "info", prdcfg["dstype"], prdcfg["prdname"], ["csv"], timeinfo=None
        )

        fname = savedir + fname[0]

        write_colocated_gates(dataset[prdcfg["radar"]]["coloc_dict"], fname)

        print("saved colocated gates file: " + fname)

        return fname

    if prdcfg["type"] == "PLOT_COLOCATED_GATES":
        dssavedir = prdcfg["dsname"]
        if "dssavename" in prdcfg:
            dssavedir = prdcfg["dssavename"]
        prdsavedir = prdcfg["prdname"]

        grid_size = prdcfg.get("grid_size", None)

        savedir = get_save_dir(
            prdcfg["basepath"],
            prdcfg["procname"],
            dssavedir,
            prdsavedir,
            timeinfo=None,
        )
        fname_list = make_filename(
            "map",
            prdcfg["dstype"],
            "counts",
            prdcfg["imgformat"],
        )

        for i, fname in enumerate(fname_list):
            fname_list[i] = savedir + fname

        plot_colocated_gates(dataset, prdcfg, fname_list, grid_size=grid_size)

        print("----- save to " + " ".join(fname_list))

        return fname_list

    if prdcfg["radar"] not in dataset:
        return None
    if "radar_out" not in dataset[prdcfg["radar"]]:
        return None

    prdcfg["timeinfo"] = None
    return generate_vol_products(dataset[prdcfg["radar"]], prdcfg)


def generate_time_avg_products(dataset, prdcfg):
    """
    generates time average products. Accepted product types:
        All the products of the 'VOL' dataset group

    Parameters
    ----------
    dataset : tuple
        radar objects and colocated gates dictionary

    prdcfg : dictionary of dictionaries
        product configuration dictionary of dictionaries

    Returns
    -------
    filename : str
        the name of the file created. None otherwise

    """
    prdcfg["timeinfo"] = dataset["timeinfo"]

    return generate_vol_products(dataset, prdcfg)



#!/usr/bin/env python3

import argparse
import glob
import os
import re
from collections import defaultdict
from datetime import datetime
from PIL import Image, ImageDraw, ImageFont
import math

DATE_REGEX_1 = re.compile(r"(\d{4}\d{2}\d{2}\d{2}\d{2}\d{2})")
DATE_REGEX_2 = re.compile(r"(\d{4}-\d{2}-\d{2})")
DATE_REGEX_3 = re.compile(r"(\d{4}\d{2}\d{2})")


def parse_args():
    parser = argparse.ArgumentParser(
        description="Create composite image from images with date-encoded filenames."
    )
    parser.add_argument(
        "patterns", nargs="+", help="One or more glob patterns (e.g. 'figs/**/*.png')."
    )
    parser.add_argument(
        "--split-by-date",
        default=1,
        type=int,
        help="Whether or not to split images by their date, they can then be arranged by row or column",
    )
    parser.add_argument(
        "--date-order",
        choices=["row", "column"],
        required=True,
        help="Group images with same date in a row or column.",
    )
    parser.add_argument(
        "--title", help="Optional title to place at the top of the composite image."
    )
    parser.add_argument(
        "--title-size",
        type=int,
        help="Font size for the title (in pixels). Defaults to ~3%% of image width.",
    )
    parser.add_argument(
        "--coltitles",
        help="Optional title to place at the top of every column. Separate titles should be comma-separated.",
    )
    parser.add_argument(
        "--coltitles-size",
        type=int,
        help="Font size for the column titles (in pixels). Defaults to ~2%% of image width.",
    )
    parser.add_argument(
        "--rowtitles",
        help="Optional title to place at the top of every row. Separate titles should be comma-separated.",
    )
    parser.add_argument(
        "--rowtitles-size",
        type=int,
        help="Font size for the row titles (in pixels). Defaults to ~2%% of image width.",
    )
    parser.add_argument(
        "--resize",
        type=int,
        default=100,
        help="Resize factor of the final image (in percent), default is 100%",
    )
    parser.add_argument(
        "--output",
        default="composite.jpg",
        help="Filename of the output composite image.",
    )

    return parser.parse_args()


def extract_date_from_filename(filename):
    match = DATE_REGEX_1.search(filename)
    if match:
        return match.group(1)
    match = DATE_REGEX_2.search(filename)
    if match:
        return match.group(1)
    match = DATE_REGEX_3.search(filename)
    if match:
        return match.group(1)
    return "unknown"


def collect_images(patterns, split_by_date):
    image_groups = defaultdict(list)
    for i, pattern in enumerate(patterns):
        for path in glob.glob(pattern, recursive=True):
            if os.path.isfile(path) and path.lower().endswith(
                (".png", ".jpg", ".jpeg", ".bmp", ".gif")
            ):
                if split_by_date:
                    date = extract_date_from_filename(os.path.basename(path))
                    image_groups[date].append(path)
                else:
                    image_groups[i].append(path)

    return dict(sorted(image_groups.items()))


def load_images(image_paths):
    return [Image.open(p).convert("RGB") for p in image_paths]


def create_composite(
    image_groups,
    date_order,
    title=None,
    title_size=None,
    col_titles=None,
    coltitle_size=None,
    row_titles=None,
    rowtitle_size=None,
    resize_percent=100,
):
    date_keys = sorted(image_groups)
    grouped_images = [load_images(image_groups[date]) for date in date_keys]

    max_w = max(img.width for group in grouped_images for img in group)
    max_h = max(img.height for group in grouped_images for img in group)
    pad = 10
    pad_title = 20
    # Determine layout
    if date_order == "row":
        rows = len(grouped_images)
        cols = max(len(group) for group in grouped_images)
    else:
        cols = len(grouped_images)
        rows = max(len(group) for group in grouped_images)

    # Font setup
    canvas_w = cols * (max_w + pad) + pad
    auto_font_size = max(16, int(canvas_w * 0.03))
    font_size = int(title_size) if title_size else auto_font_size
    font = ImageFont.load_default(font_size)

    auto_font_size = max(14, int(canvas_w * 0.02))
    coltitle_size = int(coltitle_size) if coltitle_size else auto_font_size
    rowtitle_size = int(rowtitle_size) if rowtitle_size else auto_font_size
    rowfont = ImageFont.load_default(rowtitle_size)
    colfont = ImageFont.load_default(coltitle_size)

    draw_dummy = ImageDraw.Draw(Image.new("RGB", (1, 1)))
    row_title_width = (
        max(draw_dummy.textlength(title, font=rowfont) for title in row_titles) + pad
        if row_titles
        else 0
    )
    col_title_height = coltitle_size + pad_title if col_titles else 0
    main_title_height = font_size + pad_title if title else 0

    # Canvas dimensions
    canvas_w += row_title_width
    canvas_w = int(canvas_w)
    canvas_h = int(rows * (max_h + pad) + pad + main_title_height + col_title_height)

    composite = Image.new("RGB", (canvas_w, canvas_h), color="white")
    draw = ImageDraw.Draw(composite)

    # Draw main title
    if title:
        text_w = draw.textlength(title, font=font)
        draw.text(((canvas_w - text_w) // 2, pad), title, font=font, fill="black")

    y_offset = main_title_height + col_title_height

    for i, (date, images) in enumerate(zip(date_keys, grouped_images)):
        for j, img in enumerate(images):
            if date_order == "row":
                row_idx, col_idx = i, j
            else:
                row_idx, col_idx = j, i

            x = int(pad + row_title_width + col_idx * (max_w + pad))
            y = int(y_offset + row_idx * (max_h + pad))

            composite.paste(img.resize((max_w, max_h)), (x, y))

    # Draw column titles
    if col_titles:
        for j in range(cols):
            col_title = col_titles[j] if j < len(col_titles) else ""
            x = pad + row_title_width + j * (max_w + pad)
            y = main_title_height
            draw.text(
                (x + max_w // 2 - draw.textlength(col_title, font=font) // 2, y),
                col_title,
                font=colfont,
                fill="black",
            )

    # Draw row titles
    if row_titles:
        for i in range(rows):
            row_title = row_titles[i] if i < len(row_titles) else ""
            x = pad
            y = y_offset + i * (max_h + pad) + max_h // 2 - font_size // 2
            draw.text((x, y), row_title, font=rowfont, fill="black")
    # Resize if needed
    if resize_percent != 100:
        new_w = int(composite.width * resize_percent / 100)
        new_h = int(composite.height * resize_percent / 100)
        composite = composite.resize((new_w, new_h))

    return composite


def main():
    args = parse_args()
    print(args)
    import pdb

    pdb.set_trace()
    image_groups = collect_images(args.patterns, split_by_date=args.split_by_date)

    if not image_groups:
        print("No images found.")
        return

    if args.rowtitles:
        args.rowtitles = args.rowtitles.split(",")
    if args.coltitles:
        args.coltitles = args.coltitles.split(",")

    composite = create_composite(
        image_groups,
        args.date_order,
        title=args.title,
        title_size=args.title_size,
        row_titles=args.rowtitles,
        rowtitle_size=args.rowtitles_size,
        col_titles=args.coltitles,
        coltitle_size=args.coltitles_size,
        resize_percent=args.resize,
    )
    composite.save(args.output)
    print(f"Composite image saved to {args.output}")


if __name__ == "__main__":
    main()



import argparse
import sys
import yaml
from typing import Tuple, Any

import pyrad
from pyrad.io.config import read_config


def convert_to_yaml(input_file):
    config = read_config(input_file)
    return yaml.dump(config, sort_keys=False, allow_unicode=True)


def main():
    parser = argparse.ArgumentParser(
        description="Convert legacy config format to YAML."
    )
    parser.add_argument("input_file", help="Path to the legacy config file")
    parser.add_argument(
        "-o", "--output", help="Path to write the YAML output (default: stdout)"
    )

    args = parser.parse_args()
    try:
        yaml_output = convert_to_yaml(args.input_file)
        if args.output:
            with open(args.output, "w") as out_file:
                out_file.write(yaml_output)
        else:
            print(yaml_output)
    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()




# THIS FILE IS GENERATED FROM PYRAD_PROC SETUP.PY
short_version = '2.1.2'
version = '2.1.2'
full_version = '2.1.2'
git_revision = 'e5e4078aa69d2ad34cab31db49e22daca5e0be35'
compile_date_time = '2025-11-13 15:27'
username = 'runner'
release = True

if not release:
    version = full_version



"""
pyrad.io.config
===============

Functions for reading pyrad config files

.. autosummary::
    :toctree: generated/

    read_config
    get_num_elements
    string_to_datatype
    get_array
    get_struct
    get_array_type
    init_array

"""
import re
import os
import yaml

# Allows parsing of environment variables in YAML safe loader
env_var_pattern = re.compile(r"\$\{([^}^{]+)\}")


def env_var_constructor(loader, node):
    value = loader.construct_scalar(node)
    return os.path.expandvars(value)


yaml.SafeLoader.add_implicit_resolver("!env_var", env_var_pattern, None)
yaml.SafeLoader.add_constructor("!env_var", env_var_constructor)

# Define type mappings
SCALAR_TYPES = {
    "BYTE": int,
    "BOOL": lambda x: str(x).lower() in ("true", "1"),
    "INT": int,
    "LONG": int,
    "HEX": lambda x: int(x, 16)
    if isinstance(x, str) and x.startswith("0x")
    else int(x),
    "EXP": float,
    "FLOAT": float,
    "DOUBLE": float,
    "STRING": str,
}

ARRAY_TYPE_NAMES = {
    "BYTARR": "BYTE",
    "INTARR": "INT",
    "LONARR": "LONG",
    "HEXARR": "HEX",
    "EXPARR": "EXP",
    "FLTARR": "FLOAT",
    "DBLARR": "DOUBLE",
    "STRARR": "STRING",
}


def read_config(fname, cfg=None, defaults=None):
    """
    Read a pyrad config file.
    It can use either the classical pyrad config syntax or yaml files

    Parameters
    ----------
    fname : str
        Name of the configuration file to read.

    cfg : dict of dicts, optional
        dictionary of dictionaries containing configuration parameters where
        the new parameters will be placed

    defaults: dict of dicts, optional
        dictionary of dictionaries containing default values. If a key is
        contained in the defaults dict but not in the configuration file,
        the value from the defaults dict will be assigned

    Returns
    -------
    cfg : dict of dicts
        dictionary of dictionaries containing the configuration parameters

    """
    # if config dictionary does not exist yet create it
    if cfg is None:
        cfg = dict()

    # check if the file can be read
    try:
        with open(fname, "r", encoding="utf-8", errors="ignore") as cfgfile:
            # Figure out if it is yaml or not
            if fname.endswith(".yaml") or fname.endswith(".yml"):
                cfg_new = yaml.load(cfgfile, Loader=yaml.SafeLoader)
            else:
                cfg_new = _read_config_pyrad(cfgfile)
    except Exception:
        raise
        raise Exception("ERROR: Could not find|open config file '" + fname + "'")

    # Merge with existing dict
    cfg = merge_dicts(cfg, cfg_new)

    # if default does not exist, create it
    if defaults is None:
        defaults = dict()

    # Verify that all keys in default are in newly created config
    cfg = merge_dicts(cfg, defaults)
    return cfg


def _read_config_pyrad(cfgfile):
    lines = [
        os.path.expandvars(line).split("#")[0].strip()
        for line in cfgfile.readlines()
        if line.strip() and not line.strip().startswith("#")
    ]

    def parse_block(line, index, expected_count):
        result = {}
        count = 0
        while index < len(lines):
            line = lines[index]
            parts = line.split(maxsplit=2)

            if len(parts) < 2:
                raise ValueError(f"Invalid line at {index + 1}: '{line}'")

            key = parts[0]
            type_str = parts[1].upper()

            if type_str == "STRUCT":
                if len(parts) != 3:
                    raise ValueError(
                        f"Missing struct count at line {index + 1}: '{line}'"
                    )
                struct_count = int(parts[2])
                index += 1
                struct_data, index = parse_block(line, index, struct_count)
                result[key] = struct_data
                count += 1

            elif type_str in ARRAY_TYPE_NAMES:
                if len(parts) != 3:
                    raise ValueError(
                        f"Missing array count at line {index + 1}: '{line}'"
                    )
                arr_count = int(parts[2])
                index += 1
                arr = []
                for _ in range(arr_count):
                    if index >= len(lines):
                        raise ValueError(
                            f"Expected {arr_count} array items for '{key}'"
                        )
                    parser = SCALAR_TYPES[ARRAY_TYPE_NAMES[type_str]]
                    arr.append(parser(lines[index]))
                    index += 1
                result[key] = arr
                count += 1

            elif type_str in SCALAR_TYPES:
                if len(parts) != 3:
                    raise ValueError(
                        f"Expected value for key '{key}' at line {index + 1}"
                    )
                raw_val = parts[2]
                parser = SCALAR_TYPES[type_str]
                value = parser(raw_val)
                result[key] = value
                index += 1
                count += 1

            else:
                break  # End of current block

            if expected_count is not None and count == expected_count:
                break

        if expected_count is not None and count != expected_count:
            raise ValueError(
                f"At line {line}, expected {expected_count} elements, but parsed {count}"
            )
        return result, index

    config = {}
    index = 0
    while index < len(lines):
        line = lines[index]
        parts = line.split(maxsplit=2)
        if len(parts) < 2:
            raise ValueError(
                f"Invalid top-level declaration at line {index+1}: '{line}'"
            )
        key, type_str = parts[0], parts[1].upper()

        if type_str == "STRUCT":
            if len(parts) != 3:
                raise ValueError(f"Missing struct count at line {index + 1}: '{line}'")
            struct_count = int(parts[2])
            index += 1
            struct_data, index = parse_block(line, index, struct_count)
            config[key] = struct_data

        elif type_str in ARRAY_TYPE_NAMES:
            if len(parts) != 3:
                raise ValueError(f"Missing array count at line {index + 1}: '{line}'")
            arr_count = int(parts[2])
            index += 1
            arr = []
            for _ in range(arr_count):
                if index >= len(lines):
                    raise ValueError(f"Expected {arr_count} array items for '{key}'")
                if lines[index] == "\n" or "STRUCT" in lines[index]:
                    raise ValueError(
                        f"Parsed illegal line while reading array for {line}"
                    )
                arr.append(lines[index])
                index += 1
            config[key] = arr

        elif type_str in SCALAR_TYPES:
            if len(parts) != 3:
                raise ValueError(f"Expected value for key '{key}' at line {index + 1}")
            raw_val = parts[2]
            parser = SCALAR_TYPES[type_str]
            value = parser(raw_val)
            config[key] = value
            index += 1
        else:
            raise ValueError(f"Unknown type '{type_str}' at line {index+1}")

    if "dataSetList" in config:
        if len(config["dataSetList"]):
            # Check if all elements are in config
            for dset in config["dataSetList"]:
                if ":" in dset:
                    dset = dset.split(":")[1]
                if dset not in config:
                    raise ValueError(
                        f"Dataset {dset} specificied in dataSetList was not found in config file"
                    )
    return config


def merge_dicts(ref, defaults):
    """
    Merge two nested dictionaries recursively.

    Parameters:
    -----------
    ref : dict
        The base dictionary to be merged into.
    defaults : dict
        The dictionary whose keys and values are merged into dictionary ref.

    Returns:
    --------
    ref: dict
        The base dictionary after addition of missing keys from defaults
    """
    for key, value in defaults.items():
        if key in ref:
            if isinstance(ref[key], dict) and isinstance(value, dict):
                merge_dicts(ref[key], value)
        else:
            ref[key] = value
    return ref



# Make sure that deprecation warnings get printed by default
import warnings as _warnings
import pdb
import traceback
import sys
import os
import inspect

_warnings.simplefilter("always", DeprecationWarning)
ORANGE = "\033[38;5;208m"
RESET = "\033[0m"


def orange_warning_format(message, category, filename, lineno, file=None, line=None):
    print(f"{ORANGE}{category.__name__}:{message}\n{RESET}")


_warnings.showwarning = orange_warning_format


def warn(message, use_debug=True):
    _warnings.warn(message)
    sys.stderr.flush()
    if os.environ.get("PYRAD_DEBUG", False) and use_debug:
        trace = traceback.format_stack()
        idx_warn = (["warn" in line for line in trace]).index(True)
        print(f"\033[91mException triggered from {trace[idx_warn]}\033[0m")
        parent_frame = inspect.currentframe().f_back
        pdb.Pdb().set_trace(parent_frame)


def custom_excepthook(exc_type, exc_value, exc_traceback):
    # Print the exception
    traceback.print_exception(exc_type, exc_value, exc_traceback)

    # If the environment variable is set, enter debugger
    if os.environ.get("PYRAD_DEBUG", False):
        print("\nStarting pdb due to unhandled exception...\n")
        pdb.post_mortem(exc_traceback)


def enable_debug_on_error():
    sys.excepthook = custom_excepthook
